"use strict";

var _interopRequire = function (obj) { return obj && obj.__esModule ? obj["default"] : obj; };

module.exports = callAction;

var isFunction = _interopRequire(require("./isFunction"));

var _types = require("./types");

var ACTION_TYPE = _types.ACTION_TYPE;
var INTROSPECTION_TYPE = _types.INTROSPECTION_TYPE;
function callAction(_ref) {
  var responder = _ref.responder;
  var type = _ref.type;
  var initialValue = _ref.initialValue;
  var actionSetID = _ref.actionSetID;
  var actionID = _ref.actionID;
  var payload = _ref.payload;
  var notFoundValue = _ref.notFoundValue;
  var props = _ref.props;

  var responderFunction = findActionResponder({ responder: responder, type: type, actionSetID: actionSetID, actionID: actionID, notFoundValue: notFoundValue });
  if (responderFunction !== notFoundValue) {
    return responderFunction(initialValue, payload, { props: props });
  } else {
    return notFoundValue;
  }
}

function findActionResponder(_ref) {
  var responder = _ref.responder;
  var type = _ref.type;
  var actionSetID = _ref.actionSetID;
  var actionID = _ref.actionID;
  var notFoundValue = _ref.notFoundValue;

  if (responder[actionSetID]) {
    // Has forwarding function for entire type
    if (isFunction(responder[actionSetID])) {
      return function (initialValue, payload, props) {
        function forwardTo(responder, initialValue) {
          return callAction({
            notFoundValue: initialValue,
            responder: responder, type: type, initialValue: initialValue, actionSetID: actionSetID, actionID: actionID, payload: payload, props: props
          });
        }

        var result = responder[actionSetID](initialValue, {
          isAction: type === ACTION_TYPE,
          isIntrospection: type === INTROSPECTION_TYPE,
          type: type, actionID: actionID, payload: payload, props: props, forwardTo: forwardTo
        });
        if (typeof result === "undefined") {
          return notFoundValue;
        } else {
          return result;
        }
      };
    } else {
      var typeResponder = undefined;
      if (type === ACTION_TYPE) {
        typeResponder = responder[actionSetID];
      } else if (responder[actionSetID][type]) {
        typeResponder = responder[actionSetID][type];
      } else {
        return notFoundValue;
      }

      if (typeResponder[actionID]) {
        return typeResponder[actionID];
      }
    }
  }

  return notFoundValue;
}